<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Observer;

use Amasty\Geoip\Model\Geolocation;
use Amasty\GeoipRedirect\Helper\Data;
use FiloBlu\ExtAmastyGeoipRedirect\Model\Logger\Logger;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\UrlInterface;
use Magento\Store\Api\StoreCookieManagerInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 *
 */
class Redirect implements \Magento\Framework\Event\ObserverInterface
{

    protected $multiDomainHelper;
    protected $helper;
    protected $logger;

    /**
     * @param \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper
     * @param \FiloBlu\ExtAmastyGeoipRedirect\Model\Logger\Logger $logger
     * @param \FiloBlu\MultiDomain\Helper\Data $multiDomainHelper
     * @param \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Amasty\GeoipRedirect\Helper\Data $geoipHelper
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Amasty\Geoip\Model\Geolocation $geolocation
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Store\Api\StoreCookieManagerInterface $storeCookieManager
     */
    public function __construct(
        \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper,
        Logger $logger,
        \FiloBlu\MultiDomain\Helper\Data $multiDomainHelper,
        RemoteAddress $remoteAddress,
        ScopeConfigInterface $scopeConfig,
        Data $geoipHelper,
        UrlInterface $urlBuilder,
        StoreManagerInterface $storeManager,
        Geolocation $geolocation,
        Session $customerSession,
        StoreCookieManagerInterface $storeCookieManager
    ) {
        $this->_helper = $helper;
        $this->_logger = $logger;
        $this->_multiDomainHelper = $multiDomainHelper;
        $this->remoteAddress = $remoteAddress;
        $this->scopeConfig = $scopeConfig;
        $this->geoipHelper = $geoipHelper;
        $this->urlBuilder = $urlBuilder;
        $this->storeManager = $storeManager;
        $this->geolocation = $geolocation;
        $this->customerSession = $customerSession;
        $this->storeCookieManager = $storeCookieManager;
    }

    public function execute(Observer $observer)
    {
        $restrictionBypass = false;
        $isUserAgentToIgnore = false;

        $controller = $observer->getControllerAction();

        $request = $controller->getRequest();

        if ($request->isXmlHttpRequest()) {
            return;
        }

        $response = $controller->getResponse();

        $currentStoreId = $this->storeManager->getStore()->getId();
        $scopeStores = ScopeInterface::SCOPE_STORES;

        $redirectStatus = $this->scopeConfig->getValue(
            'amgeoipredirect/restriction/redirect_status',
            $scopeStores,
            $currentStoreId
        );

        $logEnabled = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log', $scopeStores, $currentStoreId);
        $logEnabledForCookie = $this->scopeConfig->getValue(
            'amgeoipredirect/debug/enable_log_for_cookie',
            $scopeStores,
            $currentStoreId
        );
        $logEnabledForIps = $this->scopeConfig->getValue(
            'amgeoipredirect/debug/enable_log_for_ip',
            $scopeStores,
            $currentStoreId
        );

        $ipRestriction = $this->scopeConfig->getValue('amgeoipredirect/restriction/ip_restriction');
        $currentIp = $this->remoteAddress->getRemoteAddress();
        $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
        $fakeCountry = $this->scopeConfig->getValue(
            'amgeoipredirect/debug/fake_country',
            $scopeStores,
            $currentStoreId
        );

        if ($logEnabled && $logEnabledForIps && !$currentIp) {
            $this->_logger->info("Can't get ip");
        }

        if ($logEnabled && $logEnabledForIps) {
            if (strpos($logEnabledForIps, $currentIp) != false || $logEnabledForIps == $currentIp) {
            } else {
                $logEnabled = false;
            }
        }

        if ($logEnabled && $logEnabledForCookie && !$this->_helper->getCanLogCookie()) {
            $logEnabled = false;
        }

        $this->redirectAllowed = $this->_helper->applyLogic(
            $request,
            $logEnabled,
            $this->urlBuilder->getCurrentUrl(),
            $this->_logger
        );

        if ($logEnabled) {
            $this->_logger->info(
                ' #### START REQUEST #### ' . $this->urlBuilder->getCurrentUrl(
                ) . ' controller->' . $request->getControllerModule()
            );
        }


        if ($logEnabled) {
            $this->_logger->info('Current Ip: ' . $currentIp);

            if ($fakeIp) {
                $this->_logger->info('Fake Ip: ' . $fakeIp);
            }

            if ($fakeCountry) {
                $this->_logger->info('Fake Country: ' . $fakeCountry);
            }
        }

        if ($fakeIp) {
            $currentIp = $fakeIp;
        }

        if (!empty($ipRestriction)) {
            $ipRestriction = array_map('rtrim', explode(PHP_EOL, $ipRestriction));
            foreach ($ipRestriction as $ip) {
                if ($currentIp && $currentIp == $ip) {
                    if ($logEnabled) {
                        $this->_logger->info('No Redirect For Ip restriction ' . $ip);
                    }
                    $restrictionBypass = true;
                }
            }
        }
        $isApi = $request->getControllerModule() == 'Mage_Api';
        if ($isApi || !$this->geoipHelper->isModuleOutputEnabled('Amasty_Geoip')) {
            if ($logEnabled) {
                $this->_logger->info('No Redirect For No Output enabled Module or API Call');
            }
            $restrictionBypass = true;
        }

        $userAgent = $request->getHeader('USER_AGENT');
        $userAgentsIgnore = $this->scopeConfig->getValue('amgeoipredirect/restriction/user_agents_ignore');
        if (!empty($userAgentsIgnore)) {
            $userAgentsIgnore = explode(',', $userAgentsIgnore);
            $userAgentsIgnore = array_map('trim', $userAgentsIgnore);
            foreach ($userAgentsIgnore as $agent) {
                if ($userAgent && $agent && stripos($userAgent, $agent) !== false) {
                    if ($logEnabled) {
                        $this->_logger->info('No Redirect For Not Allowed User agent ' . $userAgent);
                    }
                    $restrictionBypass = true;
                    $isUserAgentToIgnore = true;
                }
            }
        }


        if ($logEnabled) {
            if ($this->redirectAllowed) {
                $this->_logger->info('redirect is allowed');
            } else {
                $this->_logger->info('redirect is not allowed');
            }
        }

        if ($this->scopeConfig->getValue(
                'amgeoipredirect/general/enable',
                $scopeStores,
                $currentStoreId
            ) && $this->redirectAllowed && !$this->_helper->notGeolocalizedRequest($request) && !$restrictionBypass) {
            $country_header_config = $this->scopeConfig->getValue('amgeoipredirect/restriction/geolocation_header');

            if (!$request->getHeader($country_header_config)) {
                $location = $this->geolocation->locate($currentIp);
                $country = $location->getCountry();
            } else {
                $country = $request->getHeader($country_header_config);
            }

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            if ($logEnabled) {
                $this->_logger->info('Current Country: ' . $country);
            }

            if ($country) {
                $this->_helper->setLocalizedCountryCookie($country);
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/restriction/first_visit_redirect')) {
                $getAmYetRedirectStore = $this->_helper->getStoreCookie();
                $getAmYetRedirectCurrency = $this->_helper->getCurrencyCookie();
                $getAmYetRedirectUrl = $this->_helper->getUrlCookie();
            } else {
                $getAmYetRedirectStore = 0;
                $getAmYetRedirectCurrency = 0;
                $getAmYetRedirectUrl = 0;
            }

            if (!$getAmYetRedirectUrl && $this->scopeConfig->getValue('amgeoipredirect/country_url/enable_url')) {
                $urlMapping = $this->_helper->getCountryUrlMapping($currentStoreId);

                $currentUrl = $this->urlBuilder->getCurrentUrl();
                foreach ($urlMapping as $countries => $url) {
                    if (strpos($countries, $country) !== false && $url != $currentUrl) {
                        $this->_helper->setUrlCookie();
                        $url = preg_replace('/&?geolocation=[^&]*/', '', $url);
                        $response->setRedirect($url, $redirectStatus);
                        $response->sendResponse();
                        exit;
                    }
                }
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/country_store/enable_store')) {
                $redirect_always_from_slash = $this->scopeConfig->getValue(
                    'amgeoipredirect/restriction/redirect_always_from_slash'
                );
                $currentPath = trim(parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH));

                if ($getAmYetRedirectStore) {
                    $can_redirect = true;

                    if ($getAmYetRedirectStore == $this->storeManager->getStore()->getId(
                        ) && !$this->_helper->geolocationRequest($request)) {
                        $can_redirect = false;
                    }

                    if ($redirect_always_from_slash && $currentPath == '/') {
                        $can_redirect = true;
                    }

                    if ($can_redirect) {
                        $currentStoreUrl = str_replace(
                            '&amp;',
                            '&',
                            $this->storeManager->getStore($getAmYetRedirectStore)->getCurrentUrl(false)
                        );
                        $redirectStoreUrl = trim($currentStoreUrl);

                        if ($logEnabled) {
                            $this->_logger->info('Cookie is set, redirect to ->' . $redirectStoreUrl);
                        }

                        $this->storeCookieManager->setStoreCookie(
                            $this->storeManager->getStore($getAmYetRedirectStore)
                        );
                        $redirectStoreUrl = preg_replace('/&?geolocation=[^&]*/', '', $redirectStoreUrl);
                        $response->setRedirect($redirectStoreUrl, $redirectStatus);
                        $response->sendResponse();
                        exit;
                    }

                    if ($logEnabled) {
                        $this->_logger->info('Cookie is set, store is correct , no redirect required');
                    }
                } else {
                    if ($country) {
                        $this->_helper->setCountryCookie($country);

                        if ($logEnabled) {
                            $this->_logger->info("Set country cookie -> $country");
                        }
                    } else {
                        $countryid = $this->scopeConfig->getValue('general/country/default', $scopeStores);
                        $this->_helper->setCountryCookie($countryid);
                        if ($logEnabled) {
                            $this->_logger->info("Set default country cookie -> $country");
                        }
                    }

                    $allStores = $this->storeManager->getStores();

                    $currentGroup = $this->_multiDomainHelper->getCurrentGroup();

                    foreach ($allStores as $store) {
                        if (!$store->isActive(
                            ) || ($currentGroup && $currentGroup != $this->_multiDomainHelper->getCurrentGroup(
                                    $store->getWebsite()->getCode()
                                ))) {
                            continue;
                        }

                        $currentStoreUrl = str_replace('&amp;', '&', $store->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl);
                        $countries = $this->scopeConfig->getValue(
                            'amgeoipredirect/country_store/affected_countries',
                            $scopeStores,
                            $store->getId()
                        );
                        if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_between_websites')) {
                            $useMultistores = $store->getWebsiteId() == $this->storeManager->getStore()->getWebsiteId();
                        } else {
                            $useMultistores = true;
                        }

                        $can_redirect = true;

                        if ($store->getId() == $currentStoreId && !$this->_helper->geolocationRequest($request)) {
                            $can_redirect = false;
                        }

                        if ($redirect_always_from_slash && $currentPath === '/') {
                            $can_redirect = true;
                        }

                        if (
                            $country && $countries && strpos(
                                $countries,
                                $country
                            ) !== false && $can_redirect && $useMultistores
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Geolocalized, redirect to $redirectStoreUrl");
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            $redirectStoreUrl = preg_replace('/&?geolocation=[^&]*/', '', $redirectStoreUrl);
                            $response->setRedirect($redirectStoreUrl, $redirectStatus);
                            $response->sendResponse();
                            exit;
                        }

                        if (
                            $country && $countries && strpos($countries, $country) !== false && $store->getId(
                            ) == $currentStoreId
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info(
                                    'Already in the right website, set only cookie no redirect necessary'
                                );
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            break;
                        }
                    }
                }
            }

            if (!$getAmYetRedirectCurrency && $this->scopeConfig->getValue(
                    'amgeoipredirect/country_currency/enable_currency'
                )) {
                $currencyMapping = $this->_helper->getCountryCurrencyMapping($currentStoreId);
                foreach ($currencyMapping as $countries => $currency) {
                    if (strpos($countries, $country) !== false && $this->storeManager->getStore(
                        )->getCurrentCurrencyCode() != $currency) {
                        $this->_helper->setCurrencyCookie();
                        $this->storeManager->getStore()->setCurrentCurrencyCode($currency);
                    }
                }
            }
        }

        if ($this->scopeConfig->getValue(
                'amgeoipredirect/general/enable',
                $scopeStores,
                $currentStoreId
            ) && $this->redirectAllowed && !$this->_helper->notGeolocalizedRequest($request)) {
            if (!$this->scopeConfig->getValue(
                    'amgeoipredirect/landing/enable_landing',
                    $scopeStores,
                    $currentStoreId
                ) || ($this->scopeConfig->getValue(
                        'amgeoipredirect/landing/enable_landing',
                        $scopeStores,
                        $currentStoreId
                    ) && $this->scopeConfig->getValue(
                        'amgeoipredirect/landing/enable_landing_only_useragents_to_ignore',
                        $scopeStores,
                        $currentStoreId
                    ) && !$isUserAgentToIgnore)) {
                $currentUrl = $this->urlBuilder->getCurrentUrl();
                $currentUrl .= (parse_url($currentUrl, PHP_URL_QUERY) ? '&' : '?') . 'notgeolocalized=1';
                $response->setRedirect($currentUrl, $redirectStatus);
                $response->sendResponse();
                exit;
            }
        }

        if ($logEnabled) {
            $this->_logger->info(' #### END REQUEST #### ');
        }
    }
}
